--[[
The Broken Mirror Augment Swapper v0.2 by MrInfernal

ToDo:
Weapon augs

Note: 
Plane of Health merchants:
Weapon Augs: Shirlell Isnamgam

]]

---@type Mq
local mq = require('mq')
---@type ImGui
local imgui = require 'ImGui'

local myLevel = mq.TLO.Me.Level()
local myClass = mq.TLO.Me.Class.ShortName()
RoT = mq.TLO.Me.AltCurrency('Remnants of Tranquility')()
local augPrefixIndex = 1 -- For aug level
local type5Index = 1
local type9Index = 1
local sellOldAugs = true
local priestClass = {'DRU', 'SHM', 'CLR'}
local casterClass = {'ENC', 'MAG', 'NEC', 'WIZ'}
local meleeClass = {'BER', 'MNK', 'ROG'}
local hybridClass = {'BRD', 'RNG', 'BST'}
local myClassGroup = 'Tank' -- Default value. WAR, PAL, and SHD

for _, class in ipairs(priestClass) do
    if class == myClass then
        myClassGroup = 'Priest'
    end
end

for _, class in ipairs(casterClass) do
    if class == myClass then
        myClassGroup = 'Caster'
    end
end

for _, class in ipairs(meleeClass) do
    if class == myClass then
        myClassGroup = 'Melee'
    end
end

for _, class in ipairs(hybridClass) do
    if class == myClass then
        myClassGroup = 'Hybrid'
    end
end

local augPrefix = {
    '75: Elegant',
    '80: Stalwart',
    '85: Extravagant',
    '90: Glorious',
    '95: Regal',
    '100: August',
    '105: Resplendent'
}

local type5Suffix = {
    'Stability: All', --1
    'Courage: STR',
    'Resilience: STA',
    'Deftness: DEX', --4
    'Swiftness: AGI',
    'Scholarship: INT',
    'Foresight: WIS',
    'Allure: CHA' --8
}

local type9Suffix = {
    'Fulcrum: All', --1
    'Sturdy: +AC - Less HP/Mana',
    'Valiant: +HP - Less Mana', --3
    'Resourceful: +Mana - Less HP',
    'Beneficent: +Heal',
    'Vengeful: +Spell DMG - +Attack', --6
}

-- Set default type 9 according to Almar's Guide
if myClassGroup == 'Hybrid' or myClassGroup == 'Caster' then
    type9Index = 4
elseif myClassGroup == 'Melee' then
    type9Index = 6
elseif myClassGroup == 'Priest' then
    type9Index = 3
elseif myClassGroup == 'Tank' then
    type9Index = 2
end

-- Set aug level and change type 9 if needed
if myLevel >= 75 then
    augPrefixIndex = 1
elseif myLevel >= 80 and myLevel <= 84 then
    augPrefixIndex = 2
elseif myLevel >= 85 and myLevel <= 89 then
    augPrefixIndex = 3
elseif myLevel >= 90 and myLevel <= 94 then
    augPrefixIndex = 4
elseif myLevel >= 95 and myLevel <= 99 then
    augPrefixIndex = 5
elseif myLevel >= 100 and myLevel <= 104 then
    augPrefixIndex = 6
    if myClassGroup == 'Caster' or myClassGroup == 'Hybrid' then
        type9Index = 4
    end
elseif myLevel >= 105 then
    augPrefixIndex = 7
    if myClassGroup == 'Melee' then
        type9Index = 1
    elseif myClassGroup == 'Caster' then
        type9Index = 6
    elseif myClassGroup == 'Priest' then
        type9Index = 5
    end
end

local gearSlotVis = {
    'head', -- 1
    'arms',
    'leftwrist',
    'rightwrist',
    'hands',  -- 5
    'chest',
    'legs',
    'feet',  -- 8
}

local gearSlotNonVis = {
    'leftear', -- 1
    'rightear',
    'face',
    'neck',
    'shoulder',  -- 5
    'back',
    'leftfinger',
    'rightfinger',
    'waist',
    'charm',  -- 10
}

local gearSlotWep = {
    'mainhand',
    'offhand',
    'ranged'
}

VisGear = {}
NonVisGear = {}
WepGear = {}

-- Create tables for different gear types
for _, slot in ipairs(gearSlotVis) do
    table.insert(VisGear, mq.TLO.Me.Inventory(slot))
end

for _, slot in ipairs(gearSlotNonVis) do
    table.insert(NonVisGear, mq.TLO.Me.Inventory(slot))
end

for _, slot in ipairs(gearSlotWep) do
    table.insert(WepGear, mq.TLO.Me.Inventory(slot))
end

-- Get combobox values to construct aug name
local function getAugName(type, level, suffix)
    local augName = ''

    local prefix = {
        'Elegant', -- 1
        'Stalwart',
        'Extravagant',
        'Glorious', -- 4
        'Regal',
        'August',
        'Resplendent' -- 7
    }
    
    local suffix5 = {
        'Stability', --1
        'Courage',
        'Resilience',
        'Deftness', --4
        'Swiftness',
        'Scholarship',
        'Foresight',
        'Allure' --8
    }

    local suffix9 = {
        'the Fulcrum', --1
        'the Sturdy',
        'the Valiant', --3
        'the Resourceful',
        'the Beneficent',
        'the Vengeful', --6
    }

    if type == 5 then
        augName = prefix[level] .. ' Sulstone of ' .. suffix5[suffix]
    elseif type == 9 then
        augName = prefix[level] .. ' Sulstone of ' .. suffix9[suffix]
    end

    return augName
end

-- Check gear for available aug slots
-- Will check if proper type and if already slotted
local function countAugs(table, type, level, suffix)

    local count = 0
    local cost = 0
    local name = getAugName(type, level, suffix)

    --[['weps' 
        1h/shield: aug2: 5, aug4: 9
           2h:     aug3: 5, aug5: 9]]

    if type == 5 then
        if table == 'armor' then
            for _, gear in ipairs(VisGear) do
                if gear then
                    if gear.AugSlot(1).Name() ~= name and gear.AugSlot(1).Type() == 5 then
                        count = count +1
                    end
                end
            end
            for _, gear in ipairs(NonVisGear) do
                if gear then
                    if gear.AugSlot(2).Name() ~= name and gear.AugSlot(2).Type() == 5 then
                        count = count +1
                    end
                end
            end
        end
    end

    if type == 9 then
        if table == 'armor' then
            for _, gear in ipairs(VisGear) do
                if gear then
                    if gear.AugSlot(2).Name() ~= name and gear.AugSlot(2).Type() == 9 then
                        count = count +1
                    end
                end
            end
            for _, gear in ipairs(NonVisGear) do
                if gear then
                    if gear.AugSlot(4).Name() ~= name and gear.AugSlot(4).Type() == 9 then
                        count = count +1
                    end
                end
            end
        end
    end

    if level == 1 then
        cost = count * 50
    elseif level == 2 then
        cost = count * 90
    elseif level == 3 then
        cost = count * 190
    elseif level == 4 then
        cost = count * 380
    elseif level == 5 then
        cost = count * 630
    elseif level == 6 then
        cost = count * 940
    elseif level == 7 then
        cost = count * 1250
    end

    return count, name, cost
end

-- Allow checking count and cost of requested aug
local function checkAugs(table, type, level, suffix)

    local count, name, cost = countAugs(table, type, level, suffix)

    if count == 0 then
        printf('No slots found for %s.\nEnsure that you have type %d slots.', name, type)
    else
        printf('Need to purchase %d %s for %d Remnants.', count, name, cost)
    end
end

local function doAugs(table, type, level, suffix, sell)
    --Get a count and cost of the aug
    local myLevel = mq.TLO.Me.Level()
    local levelCheck = true
    local count, name, cost = countAugs(table, type, level, suffix)
    local solvCount = count
    local invCheck = 0
    local solvCheck = 0
    local gearForAugs = {}
    local removedAugs = {}

    local stop = false
    Delay = 10

    -- Check if high enough level for selected augs
    if level == 1 and myLevel < 75 then
        levelCheck = false
    elseif level == 2 and myLevel < 80 then
        levelCheck = false
    elseif level == 3 and myLevel < 85 then
        levelCheck = false
    elseif level == 4 and myLevel < 90 then
        levelCheck = false
    elseif level == 5 and myLevel < 95 then
        levelCheck = false
    elseif level == 6 and myLevel < 100 then
        levelCheck = false
    elseif level == 7 and myLevel < 105 then
        levelCheck = false
    end

    if not levelCheck then
        printf('Level too low for %s!', name)
        stop = true
    end

    if not stop then
        -- Set the solvent to use
        local solvent = 'Class X Augmentation Distiller'
        local solvCost = 30

        if level == 2 then
            solvent = 'Class XI Augmentation Distiller'
            solvCost = 45
        elseif level == 3 then
            solvent = 'Class XII Augmentation Distiller'
            solvCost = 55
        elseif level == 4 then
            solvent = 'Class XIII Augmentation Distiller'
            solvCost = 70
        elseif level == 5 then
            solvent = 'Class XIV Augmentation Distiller'
            solvCost = 85
        elseif level == 6 then
            solvent = 'Class XV Augmentation Distiller'
            solvCost = 110
        elseif level == 7 then
            solvent = 'Class XVI Augmentation Distiller'
            solvCost = 135
        end

        -- See if need to buy augs
        if mq.TLO.FindItemCount(name)() > 0 then
            invCheck = mq.TLO.FindItemCount(name)()
            count = count - invCheck
        end

        -- See if solvents needed
        if mq.TLO.FindItemCount(solvent)() >0 then
            solvCheck = mq.TLO.FindItemCount(solvent)()
            solvCount = solvCount - solvCheck
            if mq.TLO.FindItemCount('Perfected Augmentation Distiller')() >0 then
                solvCheck = mq.TLO.FindItemCount('Perfected Augmentation Distiller')()
                solvCount = solvCount - solvCheck
            end
            solvCost = solvCost * solvCount
        end
    end

    -- If augs are needed, then buy them
    if not stop then
        if count > 0 then
            if cost < RoT then -- If you have enough remnants
                -- Buy augs
                if not mq.TLO.Merchant.Open() then
                    mq.cmd('/tar Montelio')
                    mq.cmd('/timed 2 /invoke ${Merchant.OpenWindow}')
                    mq.cmd('/timed 3 /echo Purchasing %d %s for %d Remnants', count, name, cost)
                end
                Delay = Delay + 35
                mq.cmdf('/timed %d /invoke ${Merchant.SelectItem[=%s]}', Delay, name)
                Delay = Delay + 10
                -- Need a for loop to buy all of the augs
                for i=1, count do
                    mq.cmdf('/timed %d /invoke ${Merchant.Buy[%d]}', Delay, count)
                    Delay = Delay + 10
                end
                mq.cmdf('/timed %d /notify MerchantWnd MW_Done_Button leftmouseup', Delay)
                Delay = Delay + 15
            else
                printf('%s costs %d and you have %d! You need %d more!', name, cost, RoT, cost - RoT)
                stop = true
            end
        end
    end

    -- Buy distillers if needed
    if not stop then
        if solvCount >0 then
            if mq.TLO.Me.Cash()/1000 > solvCost then
                if not mq.TLO.Merchant.Open() then
                    mq.cmdf('/timed %d /tar Dabowe', Delay)
                    Delay = Delay + 2
                    mq.cmdf('/timed %d /invoke ${Merchant.OpenWindow}', Delay)
                    Delay = Delay + 3
                    mq.cmdf('/timed %d /echo Purchasing %d %s for %d pp!', Delay, solvCount, solvent, solvCost)
                    Delay = Delay + 15

                    mq.cmdf('/timed %d /invoke ${Merchant.SelectItem[=%d]}', Delay, solvent)
                    Delay = Delay + 10
                    mq.cmdf('/timed %d /invoke ${Merchant.Buy[%d]}', Delay, solvCount)
                    Delay = Delay + 10
                    mq.cmdf('/timed %d /notify MerchantWnd MW_Done_Button leftmouseup', Delay)
                    Delay = Delay + 15
                end
            else
                printf('%d %s costs %d! You need %d more!', solvCount, solvent, solvCost, solvCost - mq.TLO.Me.Cash()/1000)
            end
        end
    end

    -- Find items that need augs removed
    if not stop then
        local shown = false
        if type == 5 then
            if table == 'armor' then
                for _, gear in ipairs(VisGear) do
                    if gear then
                        if gear.AugSlot(1).Name() ~= name and gear.AugSlot(1).Type() == 5 then
                            mq.cmdf('/timed %d /echo Removing old augments from gear!')
                            shown = true
                            Delay = Delay + 5
                            mq.cmdf('/timed %d /removeaug "%s" "%s"', Delay, gear.AugSlot(1).Name(), gear.Name())
                            Delay = Delay + 10
                            mq.cmdf('/timed %d /if (${Window[ConfirmationDialogBox].Open}) /nomodkey /notify ConfirmationDialogBox Yes_Button leftmouseup', Delay)
                            Delay = Delay + 10
                            mq.cmdf('/timed %d /autoinventory', Delay)
                            Delay = Delay + 10
                            table.insert(gearForAugs, gear)
                            table.insert(removedAugs, gear.AugSlot(1).Name())
                        end
                    end
                end
                for _, gear in ipairs(NonVisGear) do
                    if gear then
                        if gear.AugSlot(2).Name() ~= name and gear.AugSlot(2).Type() == 5 then
                            if not shown then
                                mq.cmdf('/timed %d /echo Removing old augments from gear!')
                                shown = true
                                Delay = Delay + 5
                            end
                            mq.cmdf('/timed %d /removeaug "%s" "%s"', Delay, gear.AugSlot(2).Name(), gear.Name())
                            Delay = Delay + 10
                            mq.cmdf('/timed %d /if (${Window[ConfirmationDialogBox].Open}) /nomodkey /notify ConfirmationDialogBox Yes_Button leftmouseup', Delay)
                            Delay = Delay + 10
                            mq.cmdf('/timed %d /autoinventory', Delay)
                            Delay = Delay + 10
                            table.insert(gearForAugs, gear)
                            table.insert(removedAugs, gear.AugSlot(2).Name())
                        end
                    end
                end
            end
        end
    
        if type == 9 then
            if table == 'armor' then
                for _, gear in ipairs(VisGear) do
                    if gear then
                        if gear.AugSlot(2).Name() ~= name and gear.AugSlot(2).Type() == 9 then
                            if not shown then
                                mq.cmdf('/timed %d /echo Removing old augments from gear!')
                                shown = true
                                Delay = Delay + 5
                            end
                            mq.cmdf('/timed %d /removeaug "%s" "%s"', Delay, gear.AugSlot(2).Name(), gear.Name())
                            Delay = Delay + 10
                            mq.cmdf('/timed %d /if (${Window[ConfirmationDialogBox].Open}) /nomodkey /notify ConfirmationDialogBox Yes_Button leftmouseup', Delay)
                            Delay = Delay + 10
                            mq.cmdf('/timed %d /autoinventory', Delay)
                            Delay = Delay + 10
                            table.insert(gearForAugs, gear)
                            table.insert(removedAugs, gear.AugSlot(2).Name())
                        end
                    end
                end
                for _, gear in ipairs(NonVisGear) do
                    if gear then
                        if gear.AugSlot(4).Name() ~= name and gear.AugSlot(4).Type() == 9 then
                            if not shown then
                                mq.cmdf('/timed %d /echo Removing old augments from gear!')
                                shown = true
                                Delay = Delay + 5
                            end
                            mq.cmdf('/timed %d /removeaug "%s" "%s"', Delay, gear.AugSlot(4).Name(), gear.Name())
                            Delay = Delay + 10
                            mq.cmdf('/timed %d /if (${Window[ConfirmationDialogBox].Open}) /nomodkey /notify ConfirmationDialogBox Yes_Button leftmouseup', Delay)
                            Delay = Delay + 10
                            mq.cmdf('/timed %d /autoinventory', Delay)
                            Delay = Delay + 10
                            table.insert(gearForAugs, gear)
                            table.insert(removedAugs, gear.AugSlot(4).Name())
                        end
                    end
                end
            end
        end
    end

    -- Insert augs into gear
    if not stop then
        if #gearForAugs >0 then
            mq.cmdf('/timed %d /echo Inserting %s into %d pieces of gear!', name, #gearForAugs)
            for gear in ipairs(gearForAugs) do
                mq.cmdf('/timed %d /itemnotify "%s" leftmouseup', Delay, name) -- Pick up aug
                Delay = Delay + 10
                mq.cmdf('/timed %d /insertaug "%s"', Delay, gear()) -- Insert aug from cursor into gear
                Delay = Delay + 10
            end
        end
    end

    -- Sell old augs if selected
    if not stop then
        if sell then
            if not mq.TLO.Merchant.Open() then
                mq.cmdf('/timed %d /tar Montelio', Delay)
                Delay = Delay + 2
                mq.cmdf('/timed %d /invoke ${Merchant.OpenWindow}', Delay)
                Delay = Delay + 3
            end

            mq.cmdf('/timed %d /echo Selling %d old augs!', #removedAugs)
            Delay = Delay + 15

            for _, aug in ipairs(removedAugs) do
                mq.cmdf('/timed %d /selectitem "=%s"', Delay, aug)
                Delay = Delay + 10
                mq.cmdf('timed %d /invoke ${Merchant.Sell}', Delay)
                Delay = Delay + 10
            end
        end
    end
end

-- Update AltCurrency
local function updateRemnant()
    RoT = mq.TLO.Me.AltCurrency('Remnants of Tranquility')()
end

-- function to run the window
local function tbmAugs(open)
    -- We specify a default position/size in case there's no data in the .ini file.
    local main_viewport = imgui.GetMainViewport()
    imgui.SetNextWindowPos(main_viewport.WorkPos.x + 650, main_viewport.WorkPos.y + 20, ImGuiCond.FirstUseEver)

    -- change the window size
    imgui.SetNextWindowSize(550, 200, ImGuiCond.FirstUseEver)

    local show = false
    open, show = imgui.Begin("TBM Aug Swapper by MrInfernal", open)

    if not show then
        ImGui.End()
        return open
    end

    ImGui.PushItemWidth(ImGui.GetFontSize() * -12);

    -- Main window element area --
    imgui.Text(mq.TLO.Me.Name() .. '.  Level: ' .. myLevel .. ' ' .. mq.TLO.Me.Class.Name())
    imgui.SameLine(350)
    if imgui.Button('Go to Vendors') then
        mq.cmd('/travelto "pohealth" @ nav locyxz 1326.60, 134.65, 66.35')
    end
    imgui.SameLine()
    if imgui.Button('Stop Nav') then
        mq.cmd('/nav stop')
    end
    imgui.Text('Remants of Tranquility: %d', RoT)
    imgui.Text('\n\n')

    imgui.Text('Level     ')
    imgui.SameLine()
    augPrefixIndex = imgui.Combo(' ', augPrefixIndex, augPrefix, #augPrefix)
    imgui.SameLine()
    sellOldAugs = imgui.Checkbox(' Sell Augs', sellOldAugs)

    imgui.Text('Type 5: ')
    imgui.SameLine()
    type5Index = imgui.Combo('  ', type5Index, type5Suffix, #type5Suffix)
    imgui.SameLine()
    if imgui.Button(' Check') then
        checkAugs('armor', 5, augPrefixIndex, type5Index)
    end
    imgui.SameLine()
    if imgui.Button(' Buy') then
        doAugs('armor', 5, augPrefixIndex, type5Index, sellOldAugs)
    end

    imgui.Text('Type 9: ')
    imgui.SameLine()
    type9Index = imgui.Combo('   ', type9Index, type9Suffix, #type9Suffix)
    imgui.SameLine()
    if ImGui.Button("Check") then
        checkAugs('armor', 9, augPrefixIndex, type9Index)
    end
    imgui.SameLine()
    if ImGui.Button("Buy") then
        doAugs('armor', 9, augPrefixIndex, type9Index, sellOldAugs)
    end
    
    -- End of main window element area --
    -- Required for window elements
    imgui.Spacing()
    imgui.PopItemWidth()
    imgui.End()
    return open
end

mq.event('montelio', '#*#from Montelio#*#', updateRemnant)
mq.event('dabowe', '#*#from Dabowe#*#', updateRemnant)

local openGUI = true

ImGui.Register('TBM Aug Swapper', function()
    openGUI = tbmAugs(openGUI)
end)

while openGUI do
    mq.doevents()
    mq.delay(1000) -- equivalent to '1s'
end